/**
 * 孵化器基础数据相关逻辑
 * 作者: 李某
 * 孵化器相关逻辑 包括孵化器的增删改查
 * 
 */
import { BaseParamUpdateConfig, OrganizationParamUpdateConfig } from "../../config/checkParamConfig";
import { findEnterpriseCountByFuHuaQiUscc, groupFindEnterpriseCount } from "../../data/enterprise/enterprise";
import * as fuhuaqiData from "../../data/fuHuaQi/fuhuaqi";
import * as splitResultConfig from "../../config/splitResultConfig";
import * as configEnum from "../../config/enum";
import { checkChange, extractData } from "../../util/piecemeal";
import { BizError } from "../../util/bizError";
import { ERRORENUM } from "../../config/errorEnum";
import { eccFormParam } from "../../util/verificationParam";
import { changeEnumValue } from "../../util/verificationEnum";


/**
 * 主界面基础数据
 * 小程序端  首页的基础数据
 * 评分数据还没有
 * @param uscc 孵化器统一信用代码
 * @returns {} "incubatedAcreage", "acreageTotal", "acreagePersonalUse", "name", "lv", "institutionalNature"
 */
export async function homeData(uscc:string) {
    let dataBaseInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);

    let data:any = extractData(splitResultConfig.FuHuaQiHomeDataConfig, dataBaseInfo, false);

    data.enterpriseCount  = await findEnterpriseCountByFuHuaQiUscc(uscc) || 0;
    data.lv = changeEnumValue(configEnum.FUHUAQILV, dataBaseInfo.lv);
    data.institutionalNature = changeEnumValue(configEnum.INSTITUTIONALNATURE, dataBaseInfo.institutionalNature);

    return data;
}


/**
 * 孵化器详细数据
 * 小程序端  点击我的数据之后拉取的详细数据
 * @param uscc 孵化器统一信用代码
 * @returns data 基础数据字段
 */
export async function selectFuHuaQiBaseData(uscc:string) {
    let dataBaseInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);

    let enterpriseTotal = await findEnterpriseCountByFuHuaQiUscc(uscc); //企业总数
    
    let data:any = extractData(splitResultConfig.FuHuaQiBaseDataConfig, dataBaseInfo, false);
    data.enterpriseTotal = enterpriseTotal;
    data.numOfEntities = enterpriseTotal - dataBaseInfo.virtualEnterpriseNum; //企业总数 - 虚拟企业数量

    return data;
}


/**
 * 孵化器机构信息数据数据
 * 小程序端  点击机构信息之后拉取的详细数据
 * @param uscc 孵化器统一信用代码
 * @returns data 基础数据字段
 */
export async function selectOrganizationInfo(uscc:string) {
    let dataBaseInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);
    
    let data:any = extractData(splitResultConfig.OrganizationBaseDataConfig, dataBaseInfo, false);

    return data;
}


/**
 * 更新孵化器基础数据
 * 小程序端
 * @param uscc 孵化器统一信用代码
 * @param param 表单内容
 * @returns {isSuccess:true/false};
 */
export async function updateFuHuaQiBaseData(uscc:string, param) {
    let skipKeyList = [
        "virtualEnterpriseNum", "incubatedAcreage", "acreageTotal", "acreagePersonalUse", "hatchingGround"
    ];
    eccFormParam("更新孵化器基础数据", BaseParamUpdateConfig, param, skipKeyList);

    let baseDataInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);
    for (let key in param) {
        if (key == "hatchingGround") {
            param[key].forEach (info => {
                let {personInChargeAdd, siteAcreage, leasePrice} = info;
                if ( typeof personInChargeAdd != 'string' && personInChargeAdd) throw new BizError(ERRORENUM.参数错误, `personInChargeAdd应是string型 而不是${typeof personInChargeAdd}`);
                if ( typeof siteAcreage != 'number' && siteAcreage) throw new BizError(ERRORENUM.参数错误, `siteAcreage应是number型 而不是${typeof personInChargeAdd}`);
                if ( typeof leasePrice != 'number' && leasePrice) throw new BizError(ERRORENUM.参数错误, `leasePrice应是number型 而不是${typeof personInChargeAdd}`);
            })
        }
    }
        
    /**这里无法判断数组里面的内容是否有变化 所以 hatchingGround 直接赋值  */
    let changeList = checkChange(param, baseDataInfo);
    for (let i = 0; i < changeList.length; i++) {
        let key = changeList[i];
        baseDataInfo[key] = param[key];
    }
    baseDataInfo["hatchingGround"] = param.hatchingGround;

    await baseDataInfo.save();

    return {isSuccess:true};
}


/**
 * 更新孵化器机构信息数据
 * 小程序端
 * @param uscc 孵化器统一信用代码
 * @param param 表单内容
 * @returns {isSuccess:true/false};
 */
export async function updateOrganizationData(uscc:string, param) {
    let skipKeyList = [
        "name","virtualEnterpriseNum", "hatchingGround", "operationModelDes"//todo  写到这里了 这里和甘明讨论好哪些是必填的
        ,"logonTime", "lv", "identificationTime", "industry", "institutionalNature", "foundingTeam", "liaison", "liaisonPhone", "personInCharge", "personInChargePhone",
    ];
    eccFormParam("更新孵化器基础数据", OrganizationParamUpdateConfig, param, skipKeyList);

    let baseDataInfo = await fuhuaqiData.findFuHuaQiByUSCC(uscc);
        
    /**这里无法判断数组里面的内容是否有变化 所以 hatchingGround 直接赋值  */
    let changeList = checkChange(param, baseDataInfo);
    for (let i = 0; i < changeList.length; i++) {
        let key = changeList[i];
        if (key == "operationName" || key == "uscc") continue;
        
        baseDataInfo[key] = param[key];
    }
    baseDataInfo["foundingTeam"] = param.foundingTeam;
    if (param.foundingTeam && param.foundingTeam.length > 0) {
        baseDataInfo.foundingTeamType = param.foundingTeam[0].type;
    } else baseDataInfo.foundingTeamType = 0;

    await baseDataInfo.save();

    return {isSuccess:true};
}


/**
 * 获取孵化器基本信息列表
 * 管理后台端
 * @param lv 孵化器等级
 * @param institutionalNature 机构性质
 * @param industry 领域
 * @param operationModel 运营模式
 * @param foundingTeam 创业团队
 * @param page 页数
 * @returns 
 */
export async function fuHuaQiBaseList(lv:number, institutionalNature:number, industry:number, operationModel:number, foundingTeam:number, page:number) {
    let selectParam:any = {};
    if (lv) selectParam.lv = lv;
    if (institutionalNature) selectParam.institutionalNature = institutionalNature;
    if (industry) selectParam.industry = industry;
    if (operationModel) selectParam.operationModel = operationModel;
    if (foundingTeam) selectParam.foundingTeamType = foundingTeam;

    let enterpriseMap = await groupFindEnterpriseCount();//企业总数

    let dataBaseList = await fuhuaqiData.findFuHuaQiListByPage(selectParam, (page-1)*10 );

    let count = await fuhuaqiData.findFuHuaQiCount(selectParam);
    let dataList = [];

    dataBaseList.forEach( info => {
        let changeData:any = extractData(splitResultConfig.FuHuaQiBaseListConfig, info, true);
        // let hatchingGround = info.hatchingGround;
        // hatchingGround.forEach (info => {
        //     let {personInChargeAdd, siteAcreage, leasePrice} = info;
        //     changeData.personInChargeAdd = personInChargeAdd;
        //     changeData.siteAcreage = siteAcreage;
        //     changeData.leasePrice = leasePrice;
        // })
        let enterpriseTotal = enterpriseMap[info.uscc] || 0;
        let numOfEntities = enterpriseTotal - (info.virtualEnterpriseNum || 0 );
        numOfEntities = numOfEntities < 0 ? 0 : numOfEntities;

        changeData.enterpriseTotal = enterpriseTotal;
        changeData.numOfEntities = numOfEntities;

        changeData.lv = changeEnumValue(configEnum.FUHUAQILV, changeData.lv);
        changeData.industry = changeEnumValue(configEnum.FUHUAINDUSTRY, changeData.industry);
        changeData.institutionalNature = changeEnumValue(configEnum.INSTITUTIONALNATURE, changeData.institutionalNature);
        changeData.operationModel = changeEnumValue(configEnum.OPERATIONMODEL, changeData.operationModel);
        
        changeData.foundingTeamStr = "无";
        let foundingTeam = [];
        if (changeData.foundingTeam && changeData.foundingTeam.length) {
            changeData.foundingTeam.forEach(item => {
                let {memberName,  memberAge,  memberSchool,  des} = item;
                let type = changeEnumValue(configEnum.TEAM, item.type);//人才类型
                foundingTeam.push({
                    memberName,  memberAge,  memberSchool,  des, type,
                    memberSex : changeEnumValue(configEnum.SEX, item.memberSex),//成员性别
                    memberDEGREE : changeEnumValue(configEnum.DEGREE, item.memberDEGREE),//成员最高学历
                });
                // item.memberDEGREE = changeEnumValue(configEnum.DEGREE, item.foundingTeamStr);//成员最高学历
                /* 这里兼容了一下，团队类型数据库里存到了每一个人的信息里 以防后续要修改 */
                changeData.foundingTeamStr = type;
            });
            changeData.foundingTeam = foundingTeam;
        }

        dataList.push(changeData);
    });

    return {count, dataList};
}



/**
 * 导出符合条件的孵化器基本信息数据 
  * @param lv 孵化器等级
 * @param institutionalNature 机构性质
 * @param industry 领域
 * @param operationModel 运营模式
 * @param foundingTeam 创业团队
 * @param type 导出类型 OUTPUTTYPE 枚举
 * @returns 
 */
export async function outPutFuHuaQiBaseData(lv:number, institutionalNature:number, industry:number, operationModel:number, foundingTeam:number, type:Number) {
    let selectParam:any = {};

    if (type == configEnum.OUTPUTTYPE.当前数据) {
        if (lv) selectParam.lv = lv;
        if (operationModel) selectParam.operationModel = operationModel;
        if (institutionalNature) selectParam.institutionalNature = institutionalNature;
        if (industry) selectParam.industry = industry;
        if (foundingTeam) selectParam.foundingTeamType = foundingTeam;
    }

    let enterpriseMap = await groupFindEnterpriseCount();//拥有的企业
    let dataBaseList = await fuhuaqiData.findFuHuaQiList(selectParam);

    let keyList = [
        "name",//名称
        "operationName",//运营机构名称
        "uscc",//统一信用代码
        "enterpriseTotal",//企业总数
        "virtualEnterpriseNum",//虚拟孵化
        "numOfEntities",//实体孵化
        "logonTime",//注册时间
        "incubatedAcreage",//在孵面积
        "acreageTotal",//孵化器总面积
        "acreagePersonalUse",//孵化器自用面积
        "lv",//孵化器级别
        "identificationTime",//认定时间
        "industry",//领域
        "institutionalNature",//机构性质
        "liaison",//联系人
        "liaisonPhone",//联系电话
        "personInCharge",//负责人
        "personInChargePhone",//负责人联系电话
        "operationModel",//运营模式
        "foundingTeamType",//创业团队类型
        // "hatchingGround",//经备案孵化场地
        // "foundingTeam",//创业团队
    ];
    let titleList = [
        "名称", "运营机构名称", "统一信用代码", "企业总数", "虚拟孵化", "实体孵化",
        "注册时间", "在孵面积", "孵化器总面积", "孵化器自用面积", "孵化器级别", "认定时间",
        "领域", "机构性质", "联系人", "联系电话", "负责人", "负责人联系电话"
    ];
    
    let dataList = [];
    let maxTeamNumber = 0;//最大团队数
    let maxHatching = 0;//最大场地数
    dataBaseList.forEach( info => {
        let changeData:any = extractData(splitResultConfig.FuHuaQiBaseListConfig, info, true);//截取特定的字段

        const EnterpriseTotal = enterpriseMap[info.uscc] || 0; //企业总数
        let numOfEntities = EnterpriseTotal - (info.virtualEnterpriseNum || 0 ); //实体企业
        changeData.enterpriseTotal = EnterpriseTotal;
        changeData.numOfEntities = numOfEntities < 0 ? 0 : numOfEntities;

        changeData.lv = changeEnumValue(configEnum.FUHUAQILV, changeData.lv);
        changeData.industry = changeEnumValue(configEnum.FUHUAINDUSTRY, changeData.industry);
        changeData.institutionalNature = changeEnumValue(configEnum.INSTITUTIONALNATURE, changeData.institutionalNature);
        changeData.operationModel = changeEnumValue(configEnum.OPERATIONMODEL, changeData.operationModel);
        changeData.foundingTeamType = changeEnumValue(configEnum.TEAM, changeData.foundingTeamType) || "无";

        if (changeData.foundingTeam && changeData.foundingTeam.length ) {
            maxTeamNumber = Math.max(changeData.foundingTeam.length, maxTeamNumber);
        }
        if (changeData.hatchingGround && changeData.hatchingGround.length ) {
            maxHatching = Math.max(changeData.hatchingGround.length, maxHatching);
        }

        let subList = [];
        keyList.forEach(subInfo => {
            subList.push(info[subInfo] || '');
        });

        dataList.push(subList);
    });

    let teamKeyList = ["type", "memberName", "memberSex", "memberAge", "memberDEGREE", "memberSchool", "des"];
    let teamTitleList = ["人才类型", "成员姓名", "成员性别", "成员年龄", "成员最高学历", "毕业学校", "履历描述"];
    /**得到最大值之后生成下标 */
    for (let i = 0; i < maxTeamNumber; i++) {
        let teamNum = i+1;
        teamTitleList.forEach(title => {
            titleList.push(`成员${teamNum}-${title}`);
        });
    }
    let hatchingStartIndex = teamKeyList.length;
    /**得到最大值之后生成下标 */
    let hatchingKeyList = [ "personInChargeAdd", "siteAcreage", "leasePrice" ];
    let hatchingTitleList = ["孵化场地地址", "孵化场地面积", "孵化场地出租单价" ];
    for (let i = 0; i < maxTeamNumber; i++) {
        let teamNum = i+1;
        hatchingTitleList.forEach(title => {
            titleList.push(`场地${teamNum}-${title}`);
        });
    }
    
    dataBaseList.forEach((info, index) => {
        if (info.foundingTeam && info.foundingTeam.length ) {
            info.foundingTeam.forEach(teamInfo => {
                teamKeyList.forEach( teamKey => {
                    let addValue = teamInfo[teamKey];
                    if (teamKey == "type") addValue = changeEnumValue(configEnum.TEAM, addValue);
                    if (teamKey == "memberSex") addValue = changeEnumValue(configEnum.SEX, addValue);
                    if (teamKey == "memberDEGREE") addValue = changeEnumValue(configEnum.DEGREE, addValue);
                    dataList[index].push(addValue);
                });
            });
        }
        if (info.hatchingGround && info.hatchingGround.length ) {
            
            info.hatchingGround.forEach((hatchingInfo, hatchingIndex) => {1
                hatchingKeyList.forEach( (hatchingKey, hatchingKeyIndex) => {
                    let addIndex = hatchingStartIndex +  ( (hatchingIndex * 3 ) + (hatchingKeyIndex) )
                    dataList[index][addIndex] = hatchingInfo[hatchingKey];
                });
            });
        }
    });

    dataList.unshift(titleList)
    return dataList;
}